/* 
 * Mach Operating System
 * Copyright (c) 1991,1990,1989,1988 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS 
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they make and grant Carnegie the
 * rights to redistribute these changes.
 */
/*
 * HISTORY
 * $Log:	startup.c,v $
 * Revision 2.11  91/02/05  17:29:18  mrt
 * 	Changed to new Mach copyright
 * 	[91/02/01  16:17:40  mrt]
 * 
 * Revision 2.10  91/01/08  15:17:00  rpd
 * 	Swapin the startup thread and idle threads.
 * 	[90/11/20            rpd]
 * 
 * 	Removed swapout_thread.
 * 	[90/11/11            rpd]
 * 
 * Revision 2.9  90/10/12  18:07:34  rpd
 * 	Fixed call to thread_bind in start_kernel_threads.
 * 	Fix from Philippe Bernadat.
 * 	[90/10/10            rpd]
 * 
 * Revision 2.8  90/09/28  16:55:37  jsb
 * 	Added MACH_CLPORT support.
 * 	[90/09/28  14:05:00  jsb]
 * 
 * Revision 2.7  90/09/09  14:32:47  rpd
 * 	Fixed setup_main to call pset_sys_init.
 * 	[90/08/30            rpd]
 * 
 * Revision 2.6  90/08/27  22:03:40  dbg
 * 	Pass processor argument to choose_thread.
 * 	Rename cpu_start to cpu_launch_first_thread to avoid conflict
 * 	with processor code.
 * 	[90/07/17            dbg]
 * 
 * Revision 2.5  90/06/02  14:56:12  rpd
 * 	Start sched_thread.  On multiprocessors, start action_thread
 * 	instead of shutdown_thread.
 * 	[90/04/28            rpd]
 * 	Updated to new scheduling technology.
 * 	[90/03/26  22:19:00  rpd]
 * 
 * Revision 2.4  90/01/11  11:44:11  dbg
 * 	If multiprocessor, start shutdown thread.  If uniprocessor,
 * 	don't allocate interrupt stacks.
 * 	[89/12/19            dbg]
 * 
 * 	Add call to start other CPUs if multiprocessor.
 * 	[89/12/01            dbg]
 * 
 * Revision 2.3  89/09/08  11:26:32  dbg
 * 	Move kalloc initialization to vm_init.
 * 	[89/09/06            dbg]
 * 
 * 	Move bootstrap initialization to kern/bootstrap.c.  Initialize
 * 	device service here.
 * 	[89/08/02            dbg]
 * 
 * 	Initialize kalloc package.
 * 	[89/07/11            dbg]
 * 
 * Revision 2.2  89/08/05  16:07:22  rwd
 * 	Call mappable_time_init.
 * 	[89/08/04  18:31:35  rwd]
 * 
 * 23-May-89  David Golub (dbg) at Carnegie-Mellon University
 *	Thread creation must be done by a running thread;
 *	thread_setrun (called by thread_resume & kernel_thread) may look
 *	at current thread.
 *
 * 19-May-89  David Golub (dbg) at Carnegie-Mellon University
 *	Don't export first_task and first_thread; they don't last very
 *	long.
 *
 * 15-Aug-88  David Golub (dbg) at Carnegie-Mellon University
 *	Initial threads are now part of the kernel task, not first-task.
 *	Create the bootstrap task as a kernel task.
 *
 *  8-Aug-88  David Golub (dbg) at Carnegie-Mellon University
 *	Modified for stand-alone MACH kernel.
 *
 *  1-Jul-88  David Golub (dbg) at Carnegie-Mellon University
 *	Created.
 *
 */

/*
 *	Mach kernel startup.
 */


#include <xpr_debug.h>
#include <cpus.h>
#include <mach_host.h>
#include <mach_clport.h>

#include <mach/boolean.h>
#include <mach/machine.h>
#include <mach/task_special_ports.h>
#include <mach/vm_param.h>
#include <ipc/ipc_init.h>
#include <kern/cpu_number.h>
#include <kern/processor.h>
#include <kern/sched_prim.h>
#include <kern/task.h>
#include <kern/thread.h>
#include <kern/thread_swap.h>
#include <kern/time_out.h>
#include <kern/timer.h>
#include <kern/zalloc.h>
#include <vm/vm_kern.h>
#include <vm/vm_map.h>
#include <vm/vm_object.h>
#include <vm/vm_page.h>
#include <machine/machparam.h>
#include <machine/pmap.h>
#include <sys/version.h>

extern void	vm_mem_init();
extern void	init_timeout();
extern void	machine_init();

extern void	idle_thread();
extern void	vm_pageout();
extern void	reaper_thread();
extern void	swapin_thread();
extern void	sched_thread();
#if	MACH_CLPORT
extern void	clport_receive_thread();
#endif	MACH_CLPORT

extern void	bootstrap_create();
extern void	device_service_create();

void cpu_launch_first_thread();		/* forward */
void start_kernel_threads();	/* forward */

#if	NCPUS > 1
extern void	start_other_cpus();
extern void	action_thread();
extern int	intstack[];	/* interrupt stack for first cpu */
void		interrupt_stack_alloc();	/* forward */
#endif	NCPUS > 1

/*
 *	Running in virtual memory, on the interrupt stack.
 *	Does not return.  Dispatches initial thread.
 *
 *	Assumes that master_cpu is set.
 */
void setup_main()
{
	thread_t		startup_thread;

	panic_init();

	sched_init();
	vm_mem_init();

	/*
	 * As soon as the virtual memory system is up, we record
	 * that this CPU is using the kernel pmap.
	 */
	PMAP_ACTIVATE_KERNEL(master_cpu);

	init_timers();
	init_timeout();

#if	XPR_DEBUG
	xprbootstrap();
#endif	XPR_DEBUG

	timestamp_init();

	mapable_time_init();

	machine_init();

	machine_info.max_cpus = NCPUS;
	machine_info.memory_size = mem_size;
	machine_info.avail_cpus = 0;
	machine_info.major_version = KERNEL_MAJOR_VERSION;
	machine_info.minor_version = KERNEL_MINOR_VERSION;

#if	NCPUS > 1
	interrupt_stack_alloc();
#endif	NCPUS > 1

	/*
	 *	Initialize the IPC, task, and thread subsystems.
	 */
	ipc_bootstrap();
	task_init();
	thread_init();
	swapper_init();
	ipc_init();
#if	MACH_HOST
	pset_sys_init();
#endif	MACH_HOST
#if	MACH_CLPORT
	ipc_clport_init();
#endif	MACH_CLPORT

	/*
	 *	Kick off the time-out driven routines by calling
	 *	them the first time.
	 */
	recompute_priorities();
	compute_mach_factor();
	
	/*
	 *	Create a kernel thread to start the other kernel
	 *	threads.  Thread_resume (from kernel_thread) calls
	 *	thread_setrun, which may look at current thread;
	 *	we must avoid this, since there is no current thread.
	 */

	/*
	 * Create the thread, and point it at the routine.
	 */
	(void) thread_create(kernel_task, &startup_thread);
	thread_start(startup_thread, start_kernel_threads);

	/*
	 * Give it a kernel stack.
	 */
	thread_doswapin(startup_thread);
	thread_swappable(startup_thread, FALSE);

	/*
	 * Pretend it is already running, and resume it.
	 * Since it looks as if it is running, thread_resume
	 * will not try to put it on the run queues.
	 *
	 * We can do all of this without locking, because nothing
	 * else is running yet.
	 */
	startup_thread->state |= TH_RUN;
	(void) thread_resume(startup_thread);

	/*
	 * Start the thread.
	 */
	cpu_launch_first_thread(startup_thread);
	/*NOTREACHED*/
}

/*
 * Now running in a thread.  Create the rest of the kernel threads
 * and the bootstrap task.
 */
void start_kernel_threads()
{
	register int	i;

	/*
	 *	Create the idle threads and the other
	 *	service threads.
	 */
	for (i = 0; i < NCPUS; i++) {
	    if (machine_slot[i].is_cpu) {
		thread_t	th;

		(void) thread_create(kernel_task, &th);
		thread_bind(th, cpu_to_processor(i));
		thread_start(th, idle_thread);
		thread_doswapin(th);
		thread_swappable(th, FALSE);
		(void) thread_resume(th);
	    }
	}

	(void) kernel_thread(kernel_task, reaper_thread);
	(void) kernel_thread(kernel_task, swapin_thread);
	(void) kernel_thread(kernel_task, sched_thread);
#if	MACH_CLPORT
	(void) kernel_thread(kernel_task, clport_receive_thread);
#endif	MACH_CLPORT

#if	NCPUS > 1
	/*
	 *	Create the shutdown thread.
	 */
	(void) kernel_thread(kernel_task, action_thread);

	/*
	 *	Allow other CPUs to run.
	 */
	start_other_cpus();
#endif	NCPUS > 1

	/*
	 *	Create the device service.
	 */
	device_service_create();

	/*
	 *	Start the user bootstrap.
	 */
	bootstrap_create();

#if	XPR_DEBUG
	xprinit();		/* XXX */
#endif	XPR_DEBUG

	/*
	 *	Become the pageout daemon.
	 */
	vm_pageout();
	/*NOTREACHED*/
}

#if	NCPUS > 1
void
interrupt_stack_alloc()
{
	register int		i;
	register vm_offset_t	cur_stack;
	vm_offset_t		end_stack;

	/*
	 *	Create interrupt stacks for the other processors
	 *	(the first processor up uses a preallocated stack).
	 */
	cur_stack = kmem_alloc(kernel_map, NCPUS*INTSTACK_SIZE);
	end_stack = cur_stack + round_page(NCPUS*INTSTACK_SIZE);
	for (i = 0; i < NCPUS; i++) {
	    if (machine_slot[i].is_cpu) {
		if (i != master_cpu) {
		    interrupt_stack[i] = cur_stack;
		    cur_stack += INTSTACK_SIZE;
		}
		else {
		    interrupt_stack[i] = (vm_offset_t)&intstack[0];
		}
	    }
	    else {
		interrupt_stack[i] = (vm_offset_t)0;
	    }
	}

	/*
	 *	Free any stacks we do not need.
	 */
	cur_stack = round_page(cur_stack);
	if (end_stack != cur_stack)
	    kmem_free(kernel_map, cur_stack, end_stack - cur_stack);
}

extern void	slave_machine_init();

void slave_main()
{
	slave_machine_init();

	cpu_launch_first_thread(THREAD_NULL);
}
#endif	NCPUS > 1

/*
 *	Start up the first thread on a CPU.
 *	First thread is specified for the master CPU.
 */
void cpu_launch_first_thread(th)
	register thread_t	th;
{
	register int	mycpu;

	mycpu = cpu_number();

	cpu_up(mycpu);

	start_timer(&kernel_timer[mycpu]);

	/*
	 * Block all interrupts for choose_thread.
	 */
	(void) splhigh();

	if (th == THREAD_NULL)
	    th = choose_thread(cpu_to_processor(mycpu));
	if (th == THREAD_NULL)
	    panic("cpu_launch_first_thread");

	startrtclock();		/* needs an active thread */
	PMAP_ACTIVATE_KERNEL(mycpu);

	active_threads[mycpu] = th;
	th->interruptible = TRUE;
	timer_switch(&th->system_timer);

	PMAP_ACTIVATE_USER(vm_map_pmap(th->task->map), th, mycpu);

	load_context(th);
	/*NOTREACHED*/
}
