.\" 
.\" Mach Operating System
.\" Copyright (c) 1990 Carnegie-Mellon University
.\" All rights reserved.  The CMU software License Agreement specifies
.\" the terms and conditions for use and redistribution.
.\" 
.\" 
.\" HISTORY
.\" $Log:	mach_port_move_member.man,v $
.\" Revision 2.2  90/08/07  18:37:24  rpd
.\" 	Created.
.\" 
.TH mach_port_move_member 2 9/19/86
.CM 4
.SH NAME
.nf
mach_port_move_member \- moves a receive right into/out of a port set
.SH SYNOPSIS
.nf
.ft B
#include <mach/mach.h>

kern_return_t
mach_port_move_member(task, member, after)
	task_t task;
	mach_port_t member;
	mach_port_t after;
.fi
.ft P
.SH ARGUMENTS
.TP 12
.B
task
The task holding the port set and receive right.
.TP 12
.B
member
The task's name for the receive right.
.TP 12
.B
after
The task's name for the port set.
.SH DESCRIPTION
\fBmach_port_move_member\fR
moves a receive right into a port set.
If the receive right is already a member of another port set,
it is removed from that set first.
If the port set is MACH_PORT_NULL, then the receive right
is not put into a port set, but removed from its current port set.
.SH DIAGNOSTICS
.TP 25
KERN_SUCCESS
The call succeeded.
.TP 25
KERN_INVALID_TASK
\fBtask\fR was invalid.
.TP 25
KERN_INVALID_NAME
\fBmember\fR did not denote a right.
.TP 25
KERN_INVALID_RIGHT
\fBmember\fR denoted a right, but not a receive right.
.TP 25
KERN_INVALID_NAME
\fBafter\fR did not denote a right.
.TP 25
KERN_INVALID_RIGHT
\fBafter\fR denoted a right, but not a port set.
.TP 25
KERN_NOT_IN_SET
\fBafter\fR was MACH_PORT_NULL, but \fBmember\fR wasn't
currently in a port set.
.PP
The \fBmach_port_move_member\fR call is actually an RPC to \fBtask\fR,
normally a send right for a task port, but potentially any send right.
In addition to the normal diagnostic
return codes from the call's server (normally the kernel),
the call may return \fBmach_msg(2)\fR return codes.
.SH SEE ALSO
mach_port_get_set_status(2), mach_port_get_receive_status(2).
